/*
 *  D3D9.h
 *  LibAlky
 *
 *  Created by Cody Brocious on 6/5/06.
 *  Copyright 2006 Cody Brocious.  Redistribution terms are in the included LICENSE file.
 *
 */

#ifndef __ALKY_D3D9_H__
#define __ALKY_D3D9_H__

#include "Common/alky.h"

typedef struct D3DMatrix_s {
  float m[4][4];
} D3DMatrix;

typedef enum D3DDEVTYPE
{
  D3DDEVTYPE_HAL = 1,
  D3DDEVTYPE_NULLREF = 4,
  D3DDEVTYPE_REF = 2,
  D3DDEVTYPE_SW = 3,
  D3DDEVTYPE_FORCE_DWORD = 0xffffffff,
} D3DDEVTYPE, *LPD3DDEVTYPE;

typedef enum _D3DFORMAT
{
  D3DFMT_UNKNOWN              =  0,
  
  D3DFMT_R8G8B8               = 20,
  D3DFMT_A8R8G8B8             = 21,
  D3DFMT_X8R8G8B8             = 22,
  D3DFMT_R5G6B5               = 23,
  D3DFMT_X1R5G5B5             = 24,
  D3DFMT_A1R5G5B5             = 25,
  D3DFMT_A4R4G4B4             = 26,
  D3DFMT_R3G3B2               = 27,
  D3DFMT_A8                   = 28,
  D3DFMT_A8R3G3B2             = 29,
  D3DFMT_X4R4G4B4             = 30,
  D3DFMT_A2B10G10R10          = 31,
  D3DFMT_A8B8G8R8             = 32,
  D3DFMT_X8B8G8R8             = 33,
  D3DFMT_G16R16               = 34,
  D3DFMT_A2R10G10B10          = 35,
  D3DFMT_A16B16G16R16         = 36,
  
  D3DFMT_A8P8                 = 40,
  D3DFMT_P8                   = 41,
  
  D3DFMT_L8                   = 50,
  D3DFMT_A8L8                 = 51,
  D3DFMT_A4L4                 = 52,
  
  D3DFMT_V8U8                 = 60,
  D3DFMT_L6V5U5               = 61,
  D3DFMT_X8L8V8U8             = 62,
  D3DFMT_Q8W8V8U8             = 63,
  D3DFMT_V16U16               = 64,
  D3DFMT_A2W10V10U10          = 67,
  
  D3DFMT_UYVY                 = 0x59565955,
  D3DFMT_R8G8_B8G8            = 0x47424752,
  D3DFMT_YUY2                 = 0x32595559,
  D3DFMT_G8R8_G8B8            = 0x42475247,
  D3DFMT_DXT1                 = 0x31545844,
  D3DFMT_DXT2                 = 0x31545845,
  D3DFMT_DXT3                 = 0x31545846,
  D3DFMT_DXT4                 = 0x31545847,
  D3DFMT_DXT5                 = 0x31545848,
  
  D3DFMT_D16_LOCKABLE         = 70,
  D3DFMT_D32                  = 71,
  D3DFMT_D15S1                = 73,
  D3DFMT_D24S8                = 75,
  D3DFMT_D24X8                = 77,
  D3DFMT_D24X4S4              = 79,
  D3DFMT_D16                  = 80,
  
  D3DFMT_D32F_LOCKABLE        = 82,
  D3DFMT_D24FS8               = 83,
  
  
  D3DFMT_L16                  = 81,
  
  D3DFMT_VERTEXDATA           =100,
  D3DFMT_INDEX16              =101,
  D3DFMT_INDEX32              =102,
  
  D3DFMT_Q16W16V16U16         =110,
  
  D3DFMT_MULTI2_ARGB8         = 0x3154454d,
  
  // Floating point surface formats
  
  // s10e5 formats (16-bits per channel)
  D3DFMT_R16F                 = 111,
  D3DFMT_G16R16F              = 112,
  D3DFMT_A16B16G16R16F        = 113,
  
  // IEEE s23e8 formats (32-bits per channel)
  D3DFMT_R32F                 = 114,
  D3DFMT_G32R32F              = 115,
  D3DFMT_A32B32G32R32F        = 116,
  
  D3DFMT_CxV8U8               = 117,
  
  
  D3DFMT_FORCE_DWORD          =0x7fffffff
} D3DFORMAT;

typedef enum _D3DMULTISAMPLE_TYPE {
  D3DMULTISAMPLE_NONE = 0,
  D3DMULTISAMPLE_NONMASKABLE = 1,
  D3DMULTISAMPLE_2_SAMPLES = 2,
  D3DMULTISAMPLE_3_SAMPLES = 3,
  D3DMULTISAMPLE_4_SAMPLES = 4,
  D3DMULTISAMPLE_5_SAMPLES = 5,
  D3DMULTISAMPLE_6_SAMPLES = 6,
  D3DMULTISAMPLE_7_SAMPLES = 7,
  D3DMULTISAMPLE_8_SAMPLES = 8,
  D3DMULTISAMPLE_9_SAMPLES = 9,
  D3DMULTISAMPLE_10_SAMPLES = 10,
  D3DMULTISAMPLE_11_SAMPLES = 11,
  D3DMULTISAMPLE_12_SAMPLES = 12,
  D3DMULTISAMPLE_13_SAMPLES = 13,
  D3DMULTISAMPLE_14_SAMPLES = 14,
  D3DMULTISAMPLE_15_SAMPLES = 15,
  D3DMULTISAMPLE_16_SAMPLES = 16,
  D3DMULTISAMPLE_FORCE_DWORD = 0xffffffff
} D3DMULTISAMPLE_TYPE;

typedef enum D3DSWAPEFFECT
{
  D3DSWAPEFFECT_DISCARD = 1,
  D3DSWAPEFFECT_FLIP = 2,
  D3DSWAPEFFECT_COPY = 3,
  D3DSWAPEFFECT_FORCE_DWORD = 0xFFFFFFFF,
} D3DSWAPEFFECT, *LPD3DSWAPEFFECT;

typedef struct D3DPresentationParameters {
  UINT BackBufferWidth, BackBufferHeight;
  D3DFORMAT BackBufferFormat;
  UINT BackBufferCount;
  D3DMULTISAMPLE_TYPE MultiSampleType;
  DWORD MultiSampleQuality;
  D3DSWAPEFFECT SwapEffect;
  HWND hDeviceWindow;
  BOOL Windowed;
  BOOL EnableAutoDepthStencil;
  D3DFORMAT AutoDepthStencilFormat;
  DWORD Flags;
  UINT FullScreen_RefreshRateInHz;
  UINT PresentationInterval;
} D3DPresentationParameters;

typedef enum D3DTransformStateType {
  D3DTS_VIEW = 2,
  D3DTS_PROJECTION = 3,
  D3DTS_TEXTURE0 = 16,
  D3DTS_TEXTURE1 = 17,
  D3DTS_TEXTURE2 = 18,
  D3DTS_TEXTURE3 = 19,
  D3DTS_TEXTURE4 = 20,
  D3DTS_TEXTURE5 = 21,
  D3DTS_TEXTURE6 = 22,
  D3DTS_TEXTURE7 = 23,
  D3DTS_FORCE_DWORD = 0x7fffffff,
} D3DTransformStateType;

typedef enum D3DRenderStateType
{
  D3DRS_ZENABLE = 7,
  D3DRS_FILLMODE = 8,
  D3DRS_SHADEMODE = 9,
  D3DRS_ZWRITEENABLE = 14,
  D3DRS_ALPHATESTENABLE = 15,
  D3DRS_LASTPIXEL = 16,
  D3DRS_SRCBLEND = 19,
  D3DRS_DESTBLEND = 20,
  D3DRS_CULLMODE = 22,
  D3DRS_ZFUNC = 23,
  D3DRS_ALPHAREF = 24,
  D3DRS_ALPHAFUNC = 25,
  D3DRS_DITHERENABLE = 26,
  D3DRS_ALPHABLENDENABLE = 27,
  D3DRS_FOGENABLE = 28,
  D3DRS_SPECULARENABLE = 29,
  D3DRS_FOGCOLOR = 34,
  D3DRS_FOGTABLEMODE = 35,
  D3DRS_FOGSTART = 36,
  D3DRS_FOGEND = 37,
  D3DRS_FOGDENSITY = 38,
  D3DRS_RANGEFOGENABLE = 48,
  D3DRS_STENCILENABLE = 52,
  D3DRS_STENCILFAIL = 53,
  D3DRS_STENCILZFAIL = 54,
  D3DRS_STENCILPASS = 55,
  D3DRS_STENCILFUNC = 56,
  D3DRS_STENCILREF = 57,
  D3DRS_STENCILMASK = 58,
  D3DRS_STENCILWRITEMASK = 59,
  D3DRS_TEXTUREFACTOR = 60,
  D3DRS_WRAP0 = 128,
  D3DRS_WRAP1 = 129,
  D3DRS_WRAP2 = 130,
  D3DRS_WRAP3 = 131,
  D3DRS_WRAP4 = 132,
  D3DRS_WRAP5 = 133,
  D3DRS_WRAP6 = 134,
  D3DRS_WRAP7 = 135,
  D3DRS_CLIPPING = 136,
  D3DRS_LIGHTING = 137,
  D3DRS_AMBIENT = 139,
  D3DRS_FOGVERTEXMODE = 140,
  D3DRS_COLORVERTEX = 141,
  D3DRS_LOCALVIEWER = 142,
  D3DRS_NORMALIZENORMALS = 143,
  D3DRS_DIFFUSEMATERIALSOURCE = 145,
  D3DRS_SPECULARMATERIALSOURCE = 146,
  D3DRS_AMBIENTMATERIALSOURCE = 147,
  D3DRS_EMISSIVEMATERIALSOURCE = 148,
  D3DRS_VERTEXBLEND = 151,
  D3DRS_CLIPPLANEENABLE = 152,
  D3DRS_POINTSIZE = 154,
  D3DRS_POINTSIZE_MIN = 155,
  D3DRS_POINTSPRITEENABLE = 156,
  D3DRS_POINTSCALEENABLE = 157,
  D3DRS_POINTSCALE_A = 158,
  D3DRS_POINTSCALE_B = 159,
  D3DRS_POINTSCALE_C = 160,
  D3DRS_MULTISAMPLEANTIALIAS = 161,
  D3DRS_MULTISAMPLEMASK = 162,
  D3DRS_PATCHEDGESTYLE = 163,
  D3DRS_DEBUGMONITORTOKEN = 165,
  D3DRS_POINTSIZE_MAX = 166,
  D3DRS_INDEXEDVERTEXBLENDENABLE = 167,
  D3DRS_COLORWRITEENABLE = 168,
  D3DRS_TWEENFACTOR = 170,
  D3DRS_BLENDOP = 171,
  D3DRS_POSITIONDEGREE = 172,
  D3DRS_NORMALDEGREE = 173,
  D3DRS_SCISSORTESTENABLE = 174,
  D3DRS_SLOPESCALEDEPTHBIAS = 175,
  D3DRS_ANTIALIASEDLINEENABLE = 176,
  D3DRS_MINTESSELLATIONLEVEL = 178,
  D3DRS_MAXTESSELLATIONLEVEL = 179,
  D3DRS_ADAPTIVETESS_X = 180,
  D3DRS_ADAPTIVETESS_Y = 181,
  D3DRS_ADAPTIVETESS_Z = 182,
  D3DRS_ADAPTIVETESS_W = 183,
  D3DRS_ENABLEADAPTIVETESSELLATION = 184,
  D3DRS_TWOSIDEDSTENCILMODE = 185,
  D3DRS_CCW_STENCILFAIL = 186,
  D3DRS_CCW_STENCILZFAIL = 187,
  D3DRS_CCW_STENCILPASS = 188,
  D3DRS_CCW_STENCILFUNC = 189,
  D3DRS_COLORWRITEENABLE1 = 190,
  D3DRS_COLORWRITEENABLE2 = 191,
  D3DRS_COLORWRITEENABLE3 = 192,
  D3DRS_BLENDFACTOR = 193,
  D3DRS_SRGBWRITEENABLE = 194,
  D3DRS_DEPTHBIAS = 195,
  D3DRS_WRAP8 = 198,
  D3DRS_WRAP9 = 199,
  D3DRS_WRAP10 = 200,
  D3DRS_WRAP11 = 201,
  D3DRS_WRAP12 = 202,
  D3DRS_WRAP13 = 203,
  D3DRS_WRAP14 = 204,
  D3DRS_WRAP15 = 205,
  D3DRS_SEPARATEALPHABLENDENABLE = 206,
  D3DRS_SRCBLENDALPHA = 207,
  D3DRS_DESTBLENDALPHA = 208,
  D3DRS_BLENDOPALPHA = 209,
  D3DRS_FORCE_DWORD = 0x7fffffff,
} D3DRenderStateType;

typedef enum D3DCullType {
  D3DCULL_NONE                = 1,
  D3DCULL_CW                  = 2,
  D3DCULL_CCW                 = 3,
  D3DCULL_FORCE_DWORD         = 0x7fffffff,
} D3DCullType;

typedef enum D3DZBufferType {
  D3DZB_FALSE = 0,
  D3DZB_TRUE  = 1,
  D3DZB_USEW  = 2,
  D3DZB_FORCE_DWORD = 0x7fffffff
} D3DZBufferType;

typedef enum D3DSamplerStateType
{
  D3DSAMP_ADDRESSU = 1,
  D3DSAMP_ADDRESSV = 2,
  D3DSAMP_ADDRESSW = 3,
  D3DSAMP_BORDERCOLOR = 4,
  D3DSAMP_MAGFILTER = 5,
  D3DSAMP_MINFILTER = 6,
  D3DSAMP_MIPFILTER = 7,
  D3DSAMP_MIPMAPLODBIAS = 8,
  D3DSAMP_MAXMIPLEVEL = 9,
  D3DSAMP_MAXANISOTROPY = 10,
  D3DSAMP_SRGBTEXTURE = 11,
  D3DSAMP_ELEMENTINDEX = 12,
  D3DSAMP_DMAPOFFSET = 13,
  D3DSAMP_FORCE_DWORD = 0x7fffffff,
} D3DSamplerStateType;

typedef enum D3DTextureFilterType
{
  D3DTEXF_NONE = 0,
  D3DTEXF_POINT = 1,
  D3DTEXF_LINEAR = 2,
  D3DTEXF_ANISOTROPIC = 3,
  D3DTEXF_PYRAMIDALQUAD = 6,
  D3DTEXF_GAUSSIANQUAD = 7,
  D3DTEXF_FORCE_DWORD = 0x7fffffff,
} D3DTextureFilterType;

typedef enum D3DPool
{
  D3DPOOL_DEFAULT = 0,
  D3DPOOL_MANAGED = 1,
  D3DPOOL_SYSTEMMEM = 2,
  D3DPOOL_SCRATCH = 3,
  D3DPOOL_FORCE_DWORD = 0x7fffffff
} D3DPool;

#define D3DFVF_RESERVED0        0x001
#define D3DFVF_POSITION_MASK    0x400E
#define D3DFVF_XYZ              0x002
#define D3DFVF_XYZRHW           0x004
#define D3DFVF_XYZB1            0x006
#define D3DFVF_XYZB2            0x008
#define D3DFVF_XYZB3            0x00a
#define D3DFVF_XYZB4            0x00c
#define D3DFVF_XYZB5            0x00e
#define D3DFVF_XYZW             0x4002

#define D3DFVF_NORMAL           0x010
#define D3DFVF_PSIZE            0x020
#define D3DFVF_DIFFUSE          0x040
#define D3DFVF_SPECULAR         0x080

#define D3DFVF_TEXCOUNT_MASK    0xf00
#define D3DFVF_TEXCOUNT_SHIFT   8
#define D3DFVF_TEX0             0x000
#define D3DFVF_TEX1             0x100
#define D3DFVF_TEX2             0x200
#define D3DFVF_TEX3             0x300
#define D3DFVF_TEX4             0x400
#define D3DFVF_TEX5             0x500
#define D3DFVF_TEX6             0x600
#define D3DFVF_TEX7             0x700
#define D3DFVF_TEX8             0x800

#define D3DFVF_LASTBETA_UBYTE4   0x1000
#define D3DFVF_LASTBETA_D3DCOLOR 0x8000

#define D3DFVF_RESERVED2         0x6000  // 2 reserved bits

#include "D3D9_Device.h"
#include "D3D9_VertexBuffer.h"
#include "D3DX9.h"

#endif
