/*
 *  D3DGL.c
 *  LibAlky
 *
 *  Created by Cody Brocious on 6/9/06.
 *  Copyright 2006 Cody Brocious.  Redistribution terms are in the included LICENSE file.
 *
 */

#include "D3DGL.h"

uint D3DSetTransformProc(D3DTransformStateType type,
                      D3DMatrix *mat) {
  AlkyLog("D3DSetTransformProc called");
  
  switch(type) {
    case D3DTS_PROJECTION:
      glMatrixMode(GL_PROJECTION);
      break;
    case D3DTS_VIEW:
      glMatrixMode(GL_MODELVIEW);
      break;
    default:
      return 0;
  };
  
  glMultMatrixf((GLfloat *) mat->m);
  
  return 1;
}

uint D3DSetRenderStateProc(D3DRenderStateType type,
                           DWORD value) {
  AlkyLog("D3DSetRenderStateProc called");
  
  switch(type) {
    case D3DRS_AMBIENT: {
      GLfloat rgba[] = {0.0f, 0.0f, 0.0f, 1.0f};
      
      rgba[0] = 1.0f / (value & 0xFF);
      rgba[1] = 1.0f / ((value & 0xFF00) >> 8);
      rgba[2] = 1.0f / ((value & 0xFF0000) >> 16);
      
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, rgba);
      break;
    }
    case D3DRS_LIGHTING: {
      if(value)
        glEnable(GL_LIGHTING);
      else
        glDisable(GL_LIGHTING);
      break;
    }
    case D3DRS_CULLMODE: {
      switch(value) {
        case D3DCULL_NONE: {
          glDisable(GL_CULL_FACE);
          break;
        }
        case D3DCULL_CW: {
          glEnable(GL_CULL_FACE);
          glCullFace(GL_BACK);
          break;
        }
        case D3DCULL_CCW: {
          glEnable(GL_CULL_FACE);
          glCullFace(GL_FRONT);
        }
      }
      break;
    }
    case D3DRS_ZENABLE: {
      switch(value) {
        case D3DZB_USEW:
          AlkyLog("W-buffering requested.  Using Z-buffering instead");
        case D3DZB_TRUE:
          glEnable(GL_DEPTH_TEST);
          break;
        case D3DZB_FALSE:
          glDisable(GL_DEPTH_TEST);
          break;
      }
      break;
    }
    default: {
      AlkyLog("Unknown render state");
      return 0;
      }
  }
  
  return 1;
}

uint D3DSetSamplerStateProc(int sampler, D3DSamplerStateType type, DWORD value) {
  AlkyLog("D3DSetSamplerStateProc called");
  
  switch(type) {
    case D3DSAMP_MINFILTER: {
      switch(value) {
        case D3DTEXF_NONE:
        case D3DTEXF_POINT:
        case D3DTEXF_ANISOTROPIC:
        case D3DTEXF_PYRAMIDALQUAD:
        case D3DTEXF_GAUSSIANQUAD:
          value = GL_NEAREST;
          break;
        case D3DTEXF_LINEAR:
          value = GL_LINEAR;
      }
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, value);
      break;
    }
    case D3DSAMP_MAGFILTER: {
      switch(value) {
        case D3DTEXF_NONE:
        case D3DTEXF_POINT:
        case D3DTEXF_ANISOTROPIC:
        case D3DTEXF_PYRAMIDALQUAD:
        case D3DTEXF_GAUSSIANQUAD:
          value = GL_NEAREST;
          break;
        case D3DTEXF_LINEAR:
          value = GL_LINEAR_MIPMAP_LINEAR;
      }
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, value);
      break;
    }
    default: {
      AlkyLog("Unknown sampler state");
      break;
    }
  }
  return 1;
  
}

uint D3DSetFVFProc(DWORD fvf) {
  AlkyLog("D3DSetFVFProc called");
  
  return 0;
}
