/*
 *  Kernel32_Heap.c
 *  LibAlky
 *
 *  Created by Cody Brocious on 5/14/06.
 *  Copyright 2006 Cody Brocious.  Redistribution terms are in the included LICENSE file.
 *
 */

#include "Kernel32.h"

ALKYAPI HANDLE HeapCreate_W32(DWORD flOptions,DWORD dwInitialSize,DWORD dwMaximumSize) {
  AlkyLog("HeapCreate called");
  
  Heap *heap = (Heap *) malloc(sizeof(Heap));
  heap->curSize = dwInitialSize;
  heap->maxSize = dwMaximumSize;
  
  if(dwMaximumSize != 0)
    heap->loc = malloc(dwMaximumSize);
  else {
    heap->curSize = 0xFFFFF;
    heap->loc = malloc(heap->curSize);
  }
  
  heap->count = 0;
  
  return (HANDLE) heap;
}

ALKYAPI BOOL HeapDestroy_W32(HANDLE hHeap) {
  AlkyLog("HeapDestroy called");
  
  free(((Heap *) hHeap)->loc);
  free((Heap *) hHeap);
  
  return 1;
}

ALKYAPI LPVOID HeapAlloc_W32(HANDLE hHeap, DWORD dwFlags, DWORD dwBytes) {
  void *val;
  Heap *heap = (Heap *) hHeap;
  AlkyLog("HeapAlloc called.  Requesting %i bytes", dwBytes);
  
  if(heap->count != 0 && (heap->loc - heap->ptrs[0]) + dwBytes > heap->curSize) {
    // XXX: Handle this!
  }
     
  val = heap->loc;
  heap->loc += dwBytes;
  
  heap->ptrs[heap->count] = val;
  heap->sizes[heap->count] = dwBytes;
  ++heap->count;
  
  return val;
}

ALKYAPI DWORD HeapSize_W32(HANDLE hHeap, DWORD dwFlags, LPCVOID lpMem) {
  int i;
  Heap *heap = (Heap *) hHeap;
  
  AlkyLog("HeapSize called");
  
  for(i = 0; i < heap->count; ++i) {
    if(heap->ptrs[i] == lpMem)
      return heap->sizes[i];
  }
  
  return 0;
}

ALKYAPI LPVOID HeapReAlloc_W32(HANDLE hHeap, DWORD dwFlags, LPVOID lpMem, DWORD dwBytes) {
  LPVOID new;
  
  AlkyLog("HeapReAlloc called");
  
  new = HeapAlloc_W32(hHeap, dwFlags, dwBytes);
  memcpy(new, lpMem, HeapSize_W32(hHeap, 0, lpMem));
  return new;
}

ALKYAPI BOOL HeapFree_W32(HANDLE hHeap, DWORD dwFlags, LPVOID lpMem) {
  // Heap *heap = (Heap *) hHeap;
  
  AlkyLog("HeapFree called");
  
  // heap->loc -= HeapSize_W32(hHeap, dwFlags, lpMem);
  // --heap->count;

  return 1;
}

// STUB(LPVOID, HeapAlloc, (HANDLE hHeap,DWORD dwFlags,DWORD dwBytes), 0);
// STUB(BOOL, HeapDestroy, (HANDLE hHeap), 0);
// STUB(HANDLE, HeapCreate, (DWORD flOptions,DWORD dwInitialSize,DWORD dwMaximumSize), 0);
// STUB(BOOL, HeapFree, (HANDLE hHeap,DWORD dwFlags,LPVOID lpMem), 0);
// STUB(LPVOID, HeapReAlloc, (HANDLE hHeap,DWORD dwFlags,LPVOID lpMem,DWORD dwBytes), 0);
// STUB(DWORD, HeapSize, (HANDLE hHeap,DWORD dwFlags,LPCVOID lpMem), 0);
