/*
 *  Kernel32_IO.c
 *  LibAlky
 *
 *  Created by Cody Brocious on 5/15/06.
 *  Copyright 2006 Cody Brocious.  Redistribution terms are in the included LICENSE file.
 *
 */

#include "Kernel32.h"

#define ERROR_FILE_NOT_FOUND 2
#define ERROR_ACCESS_DENIED 5

ALKYAPI HANDLE GetStdHandle_W32(DWORD nStdHandle) {
  AlkyLog("GetStdHandle called");
  
  switch(nStdHandle) {
  case -10:
    return stdin;
  case -11:
    return stdout;
  case -12:
    return stderr;
  }
  return (HANDLE) 0;
}

ALKYAPI DWORD GetFileType_W32(HANDLE hFile) {
  AlkyLog("GetFileType called");
  
  if(hFile == stdin || hFile == stdout || hFile == stderr)
    return FILE_TYPE_CHAR;
  return FILE_TYPE_UNKNOWN;
}

ALKYAPI BOOL DeleteFileA_W32(LPCTSTR lpFileName) {
  AlkyLog("DeleteFileA called, deleting `%s'", lpFileName);
  if( remove((char *) lpFileName) == -1 ) {
    switch( errno ) {
      case EACCES:
        SetLastError_W32( ERROR_ACCESS_DENIED );
        break;
      case ENOENT:
        SetLastError_W32( ERROR_FILE_NOT_FOUND );
        break;
      default:
        SetLastError_W32( errno );
    }
    return 0;
  }
  return 1;
}

ALKYAPI BOOL DeleteFileW_W32( LPCTSTR lpFileName ) {
  AlkyLog( "(UNIMPLEMENTED UNICODE) DeleteFileW called, deleting `%s'", lpFileName );
  return DeleteFileA_W32( lpFileName );
}


STUB(BOOL, FileTimeToLocalFileTime, (const FILETIME *lpFileTime,LPFILETIME lpLocalFileTime), 0);
STUB(BOOL, FindClose, (HANDLE hFindFile), 0);
STUB(BOOL, FlushFileBuffers, (HANDLE hFile), 0);
STUB(BOOL, GetFileAttributesExA, (LPCSTR lpFileName,GET_FILEEX_INFO_LEVELS fInfoLevelId,LPVOID lpFileInformation), 0);
STUB(DWORD, GetLogicalDriveStringsA, (DWORD nBufferLength,LPSTR lpBuffer), 0);
STUB(UINT, GetDriveTypeA, (LPCSTR lpRootPathName), 0);
STUB(BOOL, FileTimeToSystemTime, (const FILETIME *lpFileTime,LPSYSTEMTIME lpSystemTime), 0);
STUB(BOOL, FindNextFileA, (HANDLE hFindFile,LPWIN32_FIND_DATAA lpFindFileData), 0);
STUB(LONG, CompareFileTime, (const FILETIME *lpFileTime1,const FILETIME *lpFileTime2), 0);
STUB(HANDLE, FindFirstFileA, (LPCSTR lpFileName,LPWIN32_FIND_DATAA lpFindFileData), 0);
STUB(DWORD, GetCurrentDirectoryA, (DWORD nBufferLength,LPSTR lpBuffer), 0);
STUB(BOOL, CreateDirectoryA, (LPCSTR lpPathName,LPSECURITY_ATTRIBUTES lpSecurityAttributes), 0);
STUB(BOOL, CopyFileA, (LPCSTR lpExistingFileName,LPCSTR lpNewFileName,BOOL bFailIfExists), 0);
STUB(BOOL, WriteFile, (HANDLE hFile,LPCVOID lpBuffer,DWORD nNumberOfBytesToWrite,LPDWORD lpNumberOfBytesWritten,LPOVERLAPPED lpOverlapped), 0);
STUB(BOOL, ReadFile, (HANDLE hFile,LPVOID lpBuffer,DWORD nNumberOfBytesToRead,LPDWORD lpNumberOfBytesRead,LPOVERLAPPED lpOverlapped), 0);
STUB(DWORD, GetFileSize, (HANDLE hFile,LPDWORD lpFileSizeHigh), 0);
STUB(DWORD, SetFilePointer, (HANDLE hFile,LONG lDistanceToMove,PLONG lpDistanceToMoveHigh,DWORD dwMoveMethod), 0);
STUB(HANDLE, CreateFileA, (LPCSTR lpFileName,DWORD dwDesiredAccess,DWORD dwShareMode,LPSECURITY_ATTRIBUTES lpSecurityAttributes,DWORD dwCreationDisposition,DWORD dwFlagsAndAttributes,HANDLE hTemplateFile), 0);
STUB(BOOL, SetEndOfFile, (HANDLE hFile), 0);
STUB(BOOL, MoveFileA, (LPCSTR lpExistingFileName,LPCSTR lpNewFileName), 0);
STUB(DWORD, GetFullPathNameA, (LPCSTR lpFileName,DWORD nBufferLength,LPSTR lpBuffer,LPSTR *lpFilePart), 0);
STUB(UINT, GetSystemDirectoryW, (LPWSTR lpBuffer, UINT uSize), 0);
STUB(DWORD, GetFileAttributesA, (LPCSTR lpFileName), 0);
// STUB(HANDLE, GetStdHandle, (DWORD nStdHandle), 0);
// STUB(DWORD, GetFileType, (HANDLE hFile), 0);
// STUB(BOOL, DeleteFileA, (LPCSTR lpFileName), 0);
