import macho
import pefile

segtrans = {'.text' : ('__ALKYCODE', '__text'),
            '.textbss' : ('__ALKYTEXTBSS', '__textbss'),
            '.data' : ('__ALKYDATA', '__data'),
            '.idata': ('__ALKYIDATA', '__idata'),
            '.rdata': ('__ALKYRDATA', '__rdata'),
            '.rsrc' : ('__ALKYRSRC', '__rsrc'),
            '.tls'  : ('__ALKYTLS', '__tls'),
            }

class Convertor:
    def __init__(self, inp, out):
        pe = pefile.PE(inp)

        self.entry = pe.OPTIONAL_HEADER.ImageBase + pe.OPTIONAL_HEADER.AddressOfEntryPoint

        self.thunk = thunk = {}

        for imp in pe.DIRECTORY_ENTRY_IMPORT:
            dll, imps = imp.dll, imp.imports
            for temp in imps:
                if temp.ordinal == 17:
                    temp.name = 'InitCommonControls'
                thunk[temp.name] = (dll, temp.address)

        segments = {}

        for section in pe.sections:
            name = section.Name
            addr = section.VirtualAddress + pe.OPTIONAL_HEADER.ImageBase
            data = section.data
            virtsize = section.Misc_VirtualSize
            align = pe.OPTIONAL_HEADER.FileAlignment

            segments[name] = {}
            segments[name]['addr'] = addr
            segments[name]['data'] = data
            segments[name]['align'] = align
            segments[name]['virtsize'] = virtsize

        mo = macho.MachO()
        self.mo = mo

        for segname in segments:
            if segname not in segtrans:
                print 'Cannot translate segment', segname + '!'
                continue
            print 'Translating segment', segname
            if segtrans[segname][0] not in mo.segments:
                if segname == '.text':
                    maxprot, curprot = 0x7, 0x7
                    flags = 0x84000400L
                else:
                    maxprot, curprot = 0x7, 0x7
                    flags = 0x04000000L

                print segname, len(segments[segname]['data']), segments[segname]['virtsize']

                if segments[segname]['virtsize'] > len(segments[segname]['data']):
                    segments[segname]['data'] += '\0' * (segments[segname]['virtsize'] - len(segments[segname]['data']))

                # print segname, hex(segments[segname]['addr'] + len(segments[segname]['data'])), hex(segments[segname]['addr'] + segments[segname]['virtsize'])

                mo.segments[segtrans[segname][0]] = macho.segment(segtrans[segname][0],
                                                                  segments[segname]['addr'],
                                                                  segments[segname]['virtsize'],
                                                                  0, len(segments[segname]['data']),
                                                                  maxprot, curprot, 0)

                mo.segments[segtrans[segname][0]].addSection(macho.section(segtrans[segname][1],
                                                                           segments[segname]['addr'],
                                                                           segments[segname]['data'],
                                                                           0, 0, 0, flags,
                                                                           0, 0, vmsize=segments[segname]['virtsize']))

        self.write(mo, out)
            

    def write(self, mo, out):
        mo.write(file(out, 'w'))
