#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>

#include <i386/user_ldt.h>

typedef int (*ENTRY)(int argc, char **argv);

static unsigned char tib[0x38];

#include <architecture/i386/table.h>
#include <architecture/i386/sel.h>
#include <i386/user_ldt.h>


#include <stdlib.h>
#include <stdio.h>

/* Create and install a new LDT data descriptor with the given base address
 * and limit. This uses byte granularity, so has a maximum size of 1MB.
 */
sel_t create_ldt_entry_with_data(const void *base, size_t size)
{
  uintptr_t addr = (uintptr_t)base;
  ldt_entry_t new_desc;
  sel_t new_sel;
  int sel_idx;

  new_desc.data.limit00 = (size - 1) & 0xFFFF;
  new_desc.data.limit16 = ((size - 1) >> 16) & 0xF;
  new_desc.data.base00 = addr & 0xFFFF;
  new_desc.data.base16 = (addr >> 16) & 0xFF;
  new_desc.data.base24 = (addr >> 24) & 0xFF;
  new_desc.data.type = DESC_DATA_WRITE;
  new_desc.data.dpl = USER_PRIV;
  new_desc.data.present = 1;
  new_desc.data.stksz = DESC_CODE_32B;
  new_desc.data.granular = DESC_GRAN_BYTE;


  sel_idx = i386_set_ldt(LDT_AUTO_ALLOC, &new_desc, 1);
  if (sel_idx < 0)
    {
      perror("i386_set_ldt");
      return NULL_SEL;
    }


  new_sel.index = sel_idx;
  new_sel.rpl = USER_PRIV;
  new_sel.ti = SEL_LDT;


  return new_sel;
}

int main(int argc, char **argv, char **environ) {
  ENTRY entry = (void *) %ENTRYPOINT%;
  sel_t _fs;
  void *alkydylib;
  char path[PATH_MAX], dylibpath[PATH_MAX];

  void (*SetArgsEnviron)(int, char **, char **);

  printf("Setting up\n");

  memset(tib, 0, 0x38);

  _fs = create_ldt_entry_with_data(tib, 0x38);

  realpath(argv[0], path);

  //if(path[strlen(path) - 20 - strlen("%APPNAME%")] == '.') {
  path[strlen(path) - 16 - strlen("%APPNAME%")] = 0;
  snprintf(dylibpath, PATH_MAX, "%s/Contents/Frameworks/alky.dylib", path);
  alkydylib = dlopen(dylibpath, RTLD_NOW | RTLD_GLOBAL);
  //}
  //else
  //  alkydylib = dlopen("alky.dylib", RTLD_NOW);

  SetArgsEnviron = dlsym(alkydylib, "SetArgsEnviron");

  SetArgsEnviron(argc, argv, environ);

#define THUNK(addr, name) do { \
*((unsigned int *) addr) = (unsigned int) dlsym(alkydylib, #name); \
if(*((unsigned int *) addr) == (unsigned int) 0) \
  *((unsigned int *) addr) = (unsigned int) dlsym(alkydylib, "UnsupportedFunction"); \
} while(0)

#include "%APPNAME%_thunk.h"

  printf("Running\n");

  __asm ("mov %%ax, %%fs\n" : : "a" (_fs));
  // __asm ("sub $0x8, %esp\n");
  __asm ("call %%ecx\n" : : "c" (entry));

  return 0;
}
