import struct

class segment:
    def __init__(self, name=0, vmaddr=0, vmsize=0, fileoff=0, filesize=0, \
                 maxprot=0, initprot=0, flags=0):
        
        self.name, self.vmaddr, self.vmsize = name, vmaddr, vmsize
        self.filesize = filesize
        self.maxprot, self.initprot = maxprot, initprot
        self.flags = flags

        self.secs = []

        if self.filesize > self.vmsize:
            self.vmsize = self.filesize

    def addSection(self, section):
        section.segname = self.name
        self.secs.append(section)

    def write(self, fileoff):
        cmd, cmdsize = 0x1, 56 + (len(self.secs) * 68)
        yield struct.pack('<LL16sLLLLLLLL', cmd, cmdsize, self.name,
                          self.vmaddr, self.vmsize, fileoff,
                          self.filesize, self.maxprot, self.initprot,
                          len(self.secs), self.flags)
        for sec in self.secs:
            for d in sec.write(fileoff):
                yield d
            fileoff += sec.size

    def read(self, fp):
        self.name, self.vmaddr, self.vmsize, \
        fileoff, self.filesize, self.maxprot, \
        self.initprot, secs, self.flags = struct.unpack('<16sLLLLLLLL', fp.read(48))
        print self.name
        print fp.tell() - 48

        for i in range(secs):
            self.secs.append(section().read(fp))

        return self

class section:
    def __init__(self, name=0, addr=0, data='', align=0, reloff=0, \
                 nreloc=0, flags=0, reserved1=0, reserved2=0, \
                 vmsize=0):
        
        self.name, self.addr, self.size = name, addr, len(data)
        self.data, self.align, self.reloff = data, align, reloff
        self.nreloc, self.flags, self.reserved1 = nreloc, flags, reserved1
        self.reserved2 = reserved2

        if self.size < vmsize:
            self.size = vmsize

    def write(self, fileoff):
        yield struct.pack('<16s16sLLLLLLLLL', self.name, self.segname,
                          self.addr, self.size, fileoff, self.align,
                          self.reloff, self.nreloc, self.flags,
                          self.reserved1, self.reserved2)

    def read(self, fp):
        self.name, self.segname, self.addr, \
        self.size, self.fileoff, self.align,\
        self.reloff, self.nreloc, self.flags,\
        self.reserved1, self.reserved2 = struct.unpack('<16s16sLLLLLLLLL', fp.read(68))
        return self

class lc_load_dylinker:
    def read(self, fp, cmdsize):
        loc = struct.unpack('<L', fp.read(4))[0]
        self.name = fp.read(cmdsize - loc)
        self.name = self.name[:self.name.find('\0')]
        return self

class lc_load_dylib:
    def read(self, fp, cmdsize):
        loc = struct.unpack('<L', fp.read(4))[0]
        self.ts, self.current_version, self.compatibility_version = struct.unpack('<LLL', fp.read(12))
        self.name = fp.read(cmdsize - loc)
        self.name = self.name[:self.name.find('\0')]

        return self

class lc_symtab:
    def read(self, fp, cmdsize):
        self.symoff, self.nsyms, self.stroff, self.strsize = struct.unpack('<LLLL', fp.read(16))
        return self

class lc_dysymtab:
    def read(self, fp, cmdsize):
        fp.read(cmdsize - 8)
        return self

class lc_twolevel_hints:
    def read(self, fp, cmdsize):
        fp.read(cmdsize - 8)
        return self

class lc_unixthread:
    def read(self, fp, cmdsize):
        fp.read(cmdsize - 8)
        return self

class MachO:
    def __init__(self, isDylib=False):
        self.segments = {}
        self.segment_order = []
        self.isDylib = isDylib
        self.dylibs = []

    def write(self, fp):
        cputype = 7 # CPU_TYPE_X86
        cpusubtype = 0 # CPU_SUBTYPE_LITTLE_ENDIAN
        
        if self.isDylib:
            filetype = 6 # MH_DYLIB
        else:
            filetype = 2 # MH_EXECUTE

        flags = 0x20000

        ncmds = len(self.segments)
        cmds = self.commands()
        sizeofcmds = len(cmds)
        reserved = 0

        fp.write(struct.pack('<LLL', 0xfeedfaceL, cputype, cpusubtype))
        fp.write(struct.pack('<LLLL', filetype, ncmds, sizeofcmds, flags))
        fp.write(cmds)

        for segname in self.segments:
            for sec in self.segments[segname].secs:
                fp.write(sec.data)

    def read(self, fp):
        magic, cputype, cpusubtype, \
        filetype, ncmds, sizeofcmds, flags = struct.unpack('<LLLLLLL', fp.read(28))

        for i in range(ncmds):
            t = fp.read(8)
            cmd, cmdsize = struct.unpack('<LL', t)

            if cmd == 0x1: # LC_SEGMENT
                seg = segment().read(fp)
                self.segments[seg.name] = seg
            elif cmd == 0xe: # LC_LOAD_DYLINKER
                self.dylinker = lc_load_dylinker().read(fp, cmdsize)
            elif cmd == 0xc: # LC_LOAD_DYLIB
                self.dylibs.append(lc_load_dylib().read(fp, cmdsize))
            elif cmd == 0x2: # LC_SYMTAB
                self.symtab = lc_symtab().read(fp, cmdsize)
            elif cmd == 0xb: # LC_DYSYMTAB
                self.dysymtab = lc_dysymtab().read(fp, cmdsize)
            elif cmd == 0x16: # LC_TWOLEVEL_HINTS
                self.twolevelhints = lc_twolevel_hints().read(fp, cmdsize)
            elif cmd == 0x5: # LC_UNIXTHREAD
                self.unixthread = lc_unixthread().read(fp, cmdsize)
            else:
                print 'Unhandled command 0x%x' % cmd
                fp.read(cmdsize)

    def commands(self):
        data = ''
        fileoff = 7*4

        for segname in self.segments:
            fileoff += 56 + (len(self.segments[segname].secs) * 68)
        
        for segname in self.segments:
            for d in self.segments[segname].write(fileoff):
                data += d
            fileoff += self.segments[segname].filesize
        return data

if __name__=='__main__':
    MachO().read(file('Oblivion.mac'))
